@@ -32,6 +32,7 @@
 #include "parser_AC3.h"
 #include "parser_DTS.h"
 #include "parser_h264.h"
+#include "parser_hevc.h"
 #include "parser_MPEGAudio.h"
 #include "parser_MPEGVideo.h"
 #include "parser_Subtitle.h"
@@ -58,8 +59,7 @@ cParser::cParser(int pID, cTSStream *stream, sPtsWrap *ptsWrap, bool observePtsW
 
 cParser::~cParser()
 {
-  if (m_PesBuffer)
-    free(m_PesBuffer);
+  free(m_PesBuffer);
 }
 
 void cParser::Reset()
@@ -370,13 +370,15 @@ bool cParser::AddPESPacket(uint8_t *data, int size)
       return false;
     }
     m_PesBufferSize += m_PesBufferInitialSize / 10;
-    m_PesBuffer = (uint8_t*)realloc(m_PesBuffer, m_PesBufferSize);
-    if (m_PesBuffer == NULL)
+    uint8_t *new_buffer = (uint8_t*)realloc(m_PesBuffer, m_PesBufferSize);
+    if (new_buffer == NULL)
     {
       ERRORLOG("cParser::AddPESPacket - realloc failed");
       Reset();
       return false;
     }
+
+    m_PesBuffer = new_buffer;
   }
 
   // copy first packet of new frame to front
@@ -401,7 +403,7 @@ inline bool cParser::IsValidStartCode(uint8_t *buf, int size)
     return false;
 
   uint32_t startcode = buf[0] << 24 | buf[1] << 16 | buf[2] << 8 | buf[3];
-  if (m_Stream->Type() == stH264 || m_Stream->Type() == stMPEG2VIDEO)
+  if (m_Stream->Type() == stH264 || m_Stream->Type() == stHEVC ||m_Stream->Type() == stMPEG2VIDEO)
   {
     if (startcode >= 0x000001e0 && startcode <= 0x000001ef)
       return true;
@@ -469,6 +471,11 @@ cTSStream::cTSStream(eStreamType type, int pid, sPtsWrap *ptsWrap, bool handleSi
     m_pesParser = new cParserH264(m_pID, this, ptsWrap, true);
     m_streamContent = scVIDEO;
   }
+  else if (m_streamType == stHEVC)
+  {
+    m_pesParser = new cParserHEVC(m_pID, this, ptsWrap, true);
+    m_streamContent = scVIDEO;
+  }
   else if (m_streamType == stMPEG2AUDIO)
   {
     m_pesParser = new cParserMPEG2Audio(m_pID, this, ptsWrap, true, handleSideData);
@@ -694,7 +701,10 @@ bool cTSStream::SetVideoInformation(int FpsScale, int FpsRate, int Height, int W
       (m_Height != Height) ||
       (m_Width != Width) ||
       (m_Aspect != Aspect))
+  {
+    INFOLOG("Video stream change, pid: %d, width: %d, height: %d, aspect: %f", m_pID, Width, Height, Aspect);
     m_IsStreamChange = true;
+  }
 
   m_FpsScale        = FpsScale;
   m_FpsRate         = FpsRate;
@@ -723,7 +733,10 @@ bool cTSStream::SetAudioInformation(int Channels, int SampleRate, int BitRate, i
       (m_BlockAlign != BlockAlign) ||
       (m_BitRate != BitRate) ||
       (m_BitsPerSample != BitsPerSample))
+  {
+    INFOLOG("Audio stream change, pid: %d, channels: %d, samplerate: %d", m_pID, Channels, SampleRate);
     m_IsStreamChange = true;
+  }
 
   m_Channels        = Channels;
   m_SampleRate      = SampleRate;
