@@ -29,13 +29,15 @@
 #include "vnsi.h"
 
 #include <vdr/remux.h>
-#include <vdr/channels.h>
 #include <vdr/device.h>
 #include <vdr/receiver.h>
 #include <vdr/ci.h>
 #include <vdr/config.h>
 #include <libsi/section.h>
 #include <libsi/descriptor.h>
+#include <memory>
+#include <vector>
+#include <algorithm>
 
 // --- cLiveReceiver -------------------------------------------------
 
@@ -44,11 +46,14 @@ class cLiveReceiver: public cReceiver
 public:
   cLiveReceiver(cVideoInput *VideoInput, const cChannel *Channel, int Priority);
   virtual ~cLiveReceiver();
-  cChannel m_PmtChannel;
 
 protected:
   virtual void Activate(bool On);
+#if VDRVERSNUM >= 20301
+  virtual void Receive(const uchar *Data, int Length);
+#else
   virtual void Receive(uchar *Data, int Length);
+#endif
 
   cVideoInput *m_VideoInput;
 };
@@ -66,18 +71,20 @@ cLiveReceiver::~cLiveReceiver()
 }
 
 //void cLiveReceiver
+#if VDRVERSNUM >= 20301
+void cLiveReceiver::Receive(const uchar *Data, int Length)
+#else
 void cLiveReceiver::Receive(uchar *Data, int Length)
+#endif
 {
   m_VideoInput->Receive(Data, Length);
 }
 
-inline void cLiveReceiver::Activate(bool On)
+void cLiveReceiver::Activate(bool On)
 {
-  DEBUGLOG("activate live receiver: %d", On);
+  INFOLOG("activate live receiver: %d, pmt change: %d", On, static_cast<bool>(m_VideoInput->m_PmtChange));
   if (!On && !m_VideoInput->m_PmtChange)
-  {
-    m_VideoInput->Retune();
-  }
+    m_VideoInput->RequestRetune();
 }
 
 // --- cLivePatFilter ----------------------------------------------------
@@ -110,6 +117,7 @@ cLivePatFilter::cLivePatFilter(cVideoInput *VideoInput, const cChannel *Channel)
 
 void cLivePatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Length)
 {
+#if VDRVERSNUM < 20104
   if (Pid == 0x00)
   {
     if (Tid == 0x00)
@@ -122,7 +130,12 @@ void cLivePatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Le
       {
         if (!assoc.isNITPid())
         {
+#if VDRVERSNUM >= 20301
+          LOCK_CHANNELS_READ;
+          const cChannel *Channel =  Channels->GetByServiceID(Source(), Transponder(), assoc.getServiceId());
+#else
           const cChannel *Channel =  Channels.GetByServiceID(Source(), Transponder(), assoc.getServiceId());
+#endif
           if (Channel && (Channel == m_Channel))
           {
             int prevPmtPid = m_pmtPid;
@@ -160,7 +173,12 @@ void cLivePatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Le
     }
     m_pmtVersion = pmt.getVersionNumber();
 
+#if VDRVERSNUM >= 20301
+    LOCK_CHANNELS_READ;
+    const cChannel *Channel = Channels->GetByServiceID(Source(), Transponder(), pmt.getServiceId());
+#else
     cChannel *Channel = Channels.GetByServiceID(Source(), Transponder(), pmt.getServiceId());
+#endif
     if (Channel) {
        // Scan the stream-specific loop:
        SI::PMT::Stream stream;
@@ -389,23 +407,90 @@ void cLivePatFilter::Process(u_short Pid, u_char Tid, const u_char *Data, int Le
        pmtChannel->Modification();
        pmtChannel->SetPids(Vpid, Ppid, Vtype, Apids, Atypes, ALangs, Dpids, Dtypes, DLangs, Spids, SLangs, Tpid);
        pmtChannel->SetSubtitlingDescriptors(SubtitlingTypes, CompositionPageIds, AncillaryPageIds);
-       m_VideoInput->PmtChange(pmtChannel->Modification(CHANNELMOD_PIDS));
+       if (pmtChannel->Modification(CHANNELMOD_PIDS))
+         m_VideoInput->RequestRetune();
     }
   }
+#endif
+}
+
+// --- cDummyReceiver ----------------------------------------------------
+
+// This dummy receiver is used to detect if a recording/streaming task
+// with a higher priority has acquired the device and detached *all*
+// receivers from it.
+class cDummyReceiver : public cReceiver
+{
+public:
+  // Return a new or existing dummy receiver attached to the device.
+  static std::shared_ptr<cDummyReceiver> Create(cDevice *device);
+  virtual ~cDummyReceiver() {Detach();}
+  bool BeenDetached() {return m_BeenDetached;}
+protected:
+#if VDRVERSNUM >= 20301
+  virtual void Receive(const uchar *Data, int Length) {}
+#else
+  virtual void Receive(uchar *Data, int Length) {}
+#endif
+  virtual void Activate(bool On);
+private:
+  static std::vector<std::weak_ptr<cDummyReceiver>> s_Pool;
+  static cMutex s_PoolMutex;
+  std::atomic<bool> m_BeenDetached;
+  cDummyReceiver() : cReceiver(NULL, MINPRIORITY), m_BeenDetached(false) {}
+};
+
+std::vector<std::weak_ptr<cDummyReceiver>> cDummyReceiver::s_Pool;
+cMutex cDummyReceiver::s_PoolMutex;
+
+void cDummyReceiver::Activate(bool On)
+{
+  INFOLOG("Dummy receiver (%p) %s", this, (On)? "activated" : "deactivated");
+  if (!On)
+    m_BeenDetached = true;
+}
+
+std::shared_ptr<cDummyReceiver> cDummyReceiver::Create(cDevice *device)
+{
+  if (!device)
+    return nullptr;
+
+  cMutexLock MutexLock(&s_PoolMutex);
+  // cleanup
+  s_Pool.erase(std::remove_if(s_Pool.begin(), s_Pool.end(),
+        [](const std::weak_ptr<cDummyReceiver> &p) -> bool
+        {return !p.lock();}), s_Pool.end());
+
+  // find an active receiver for the device
+  for (auto p : s_Pool)
+  {
+    auto recv = p.lock();
+    if (!recv->BeenDetached() && recv->Device() == device)
+      return recv;
+  }
+  auto recv = std::shared_ptr<cDummyReceiver>(new cDummyReceiver);
+  if (device->AttachReceiver(recv.get()))
+  {
+    s_Pool.push_back(recv);
+    return recv;
+  }
+  return nullptr;
 }
 
 // ----------------------------------------------------------------------------
 
-cVideoInput::cVideoInput(cCondVar &condVar, cMutex &mutex, bool &retune) :
-    m_Event(condVar), m_Mutex(mutex), m_IsRetune(retune)
+cVideoInput::cVideoInput(cCondWait &event)
+  : m_PmtChange(false)
+  , m_Event(event)
+  , m_RetuneRequested(false)
 {
-  m_Device = NULL;;
+  m_Device = NULL;
+  m_camSlot = nullptr;
   m_PatFilter = NULL;
   m_Receiver = NULL;;
   m_Channel = NULL;
   m_VideoBuffer = NULL;
   m_Priority = 0;
-  m_PmtChange = false;
 }
 
 cVideoInput::~cVideoInput()
@@ -418,27 +503,58 @@ bool cVideoInput::Open(const cChannel *channel, int priority, cVideoBuffer *vide
   m_VideoBuffer = videoBuffer;
   m_Channel = channel;
   m_Priority = priority;
+  m_RetuneRequested = false;
+  m_PmtChange = false;
   m_Device = cDevice::GetDevice(m_Channel, m_Priority, false);
+  m_camSlot = nullptr;
 
   if (m_Device != NULL)
   {
-    INFOLOG("Successfully found following device: %p (%d) for receiving", m_Device, m_Device ? m_Device->CardIndex() + 1 : 0);
+    INFOLOG("Successfully found following device: %p (%d) for receiving, priority=%d", m_Device, m_Device ? m_Device->CardIndex() + 1 : 0, m_Priority);
 
     if (m_Device->SwitchChannel(m_Channel, false))
     {
-      DEBUGLOG("Creating new live Receiver");
-      //m_Device->SetCurrentChannel(m_Channel);
-      m_SeenPmt = false;
+
+      m_Device->SetCurrentChannel(m_Channel);
+
+#if VDRVERSNUM < 20104
       m_PatFilter = new cLivePatFilter(this, m_Channel);
-      m_Receiver0 = new cLiveReceiver(this, m_Channel, m_Priority);
-      m_Receiver = new cLiveReceiver(this, m_Channel, m_Priority);
-      m_Device->AttachReceiver(m_Receiver0);
       m_Device->AttachFilter(m_PatFilter);
-      cCamSlot *cs = m_Device->CamSlot();
-      if (m_Priority <= MINPRIORITY && cs)
-        cs->StartDecrypting();
+#endif
+
+      m_PmtChannel = *m_Channel;
+      m_PmtChange = true;
+
+      m_Receiver = new cLiveReceiver(this, m_Channel, m_Priority);
+      m_Receiver->SetPids(NULL);
+      m_Receiver->SetPids(&m_PmtChannel);
+      m_Receiver->AddPid(m_PmtChannel.Tpid());
+
+      m_DummyReceiver = cDummyReceiver::Create(m_Device);
+      if (!m_DummyReceiver)
+        return false;
+
+      m_camSlot = m_Device->CamSlot();
+
+#if VDRVERSNUM >= 20107
+      if (DisableScrambleTimeout && m_camSlot)
+      {
+        // HACK cDevice::AttachReceiver() doesn't start scrambling
+        // timer if priority == MINPRIORITY
+        m_Receiver->SetPriority(MINPRIORITY);
+        if (!m_Device->AttachReceiver(m_Receiver))
+          return false;
+        if (m_camSlot)
+          m_camSlot->StartDecrypting();
+        m_Receiver->SetPriority(m_Priority);
+      }
+      else
+#endif
+      {
+        if (!m_Device->AttachReceiver(m_Receiver))
+          return false;
+      }
       m_VideoBuffer->AttachInput(true);
-      Start();
       return true;
     }
   }
@@ -448,10 +564,21 @@ bool cVideoInput::Open(const cChannel *channel, int priority, cVideoBuffer *vide
 void cVideoInput::Close()
 {
   INFOLOG("close video input ...");
-  Cancel(5);
 
   if (m_Device)
   {
+    if (DisableCamBlacklist)
+    {
+      // HACK Undo ChannelCamRelations.SetChecked() - see cDevice::Action().
+      // Note: m_Device->CamSlot() returns NULL after SetChecked() is called.
+      // Use m_camSlot here.
+      if (m_Receiver && m_camSlot)
+      {
+        ChannelCamRelations.ClrChecked(m_Receiver->ChannelID(),
+                                       m_camSlot->SlotNumber());
+      }
+    }
+
     if (m_Receiver)
     {
       DEBUGLOG("Detaching Live Receiver");
@@ -462,16 +589,6 @@ void cVideoInput::Close()
       DEBUGLOG("No live receiver present");
     }
 
-    if (m_Receiver0)
-    {
-      DEBUGLOG("Detaching Live Receiver0");
-      m_Device->Detach(m_Receiver0);
-    }
-    else
-    {
-      DEBUGLOG("No live receiver present");
-    }
-
     if (m_PatFilter)
     {
       DEBUGLOG("Detaching Live Filter");
@@ -482,32 +599,19 @@ void cVideoInput::Close()
       DEBUGLOG("No live filter present");
     }
 
+    m_DummyReceiver.reset();
+
     if (m_Receiver)
     {
       DEBUGLOG("Deleting Live Receiver");
       DELETENULL(m_Receiver);
     }
 
-    if (m_Receiver0)
-    {
-      DEBUGLOG("Deleting Live Receiver0");
-      DELETENULL(m_Receiver0);
-    }
-
     if (m_PatFilter)
     {
       DEBUGLOG("Deleting Live Filter");
       DELETENULL(m_PatFilter);
     }
-
-    //m_Device->SetCurrentChannel(NULL);
-    cCamSlot *cs = m_Device->CamSlot();
-    if (m_Priority <= MINPRIORITY && cs)
-    {
-      cs->StartDecrypting();
-      if (!cs->IsDecrypting())
-        cs->Assign(NULL);
-    }
   }
   m_Channel = NULL;
   m_Device = NULL;
@@ -528,33 +632,15 @@ bool cVideoInput::IsOpen()
 
 cChannel *cVideoInput::PmtChannel()
 {
-  return &m_Receiver->m_PmtChannel;
+  return &m_PmtChannel;
 }
 
-void cVideoInput::PmtChange(int pidChange)
-{
-  if (pidChange)
-  {
-    INFOLOG("Video Input - new pmt, attaching receiver");
-    m_PmtChange = true;
-    m_Device->Detach(m_Receiver);
-    m_Receiver->SetPids(NULL);
-    m_Receiver->SetPids(&m_Receiver->m_PmtChannel);
-    m_Receiver->AddPid(m_Receiver->m_PmtChannel.Tpid());
-    m_Device->AttachReceiver(m_Receiver);
-    cCamSlot *cs = m_Device->CamSlot();
-    if (m_Priority <= MINPRIORITY && cs)
-      cs->StartDecrypting();
-    m_SeenPmt = true;
-  }
-}
-
-inline void cVideoInput::Receive(uchar *data, int length)
+inline void cVideoInput::Receive(const uchar *data, int length)
 {
   if (m_PmtChange)
   {
      // generate pat/pmt so we can configure parsers later
-     cPatPmtGenerator patPmtGenerator(&m_Receiver->m_PmtChannel);
+     cPatPmtGenerator patPmtGenerator(&m_PmtChannel);
      m_VideoBuffer->Put(patPmtGenerator.GetPat(), TS_SIZE);
      int Index = 0;
      while (uchar *pmt = patPmtGenerator.GetPmt(Index))
@@ -564,29 +650,23 @@ inline void cVideoInput::Receive(uchar *data, int length)
   m_VideoBuffer->Put(data, length);
 }
 
-void cVideoInput::Retune()
+void cVideoInput::RequestRetune()
 {
-  INFOLOG("call retune ...");
-  cMutexLock lock(&m_Mutex);
-  m_IsRetune = true;
-  m_Event.Broadcast();
+  m_RetuneRequested = true;
+  m_Event.Signal();
 }
 
-void cVideoInput::Action()
+cVideoInput::eReceivingStatus cVideoInput::ReceivingStatus()
 {
-  cTimeMs starttime;
-
-  while (Running())
+  if (!m_Device || !m_DummyReceiver)
+    return RETUNE;
+  if (m_RetuneRequested)
   {
-    if (starttime.Elapsed() > (unsigned int)PmtTimeout*1000)
-    {
-      INFOLOG("VideoInput: no pat/pmt within timeout, falling back to channel pids");
-      m_Receiver->m_PmtChannel = *m_Channel;
-      PmtChange(true);
-    }
-    if (m_SeenPmt)
-      break;
-
-    usleep(1000);
+    (void)m_Device->Receiving();  // wait for the receivers mutex
+    if (m_DummyReceiver->BeenDetached())  // DetachAllReceivers() was called
+      return CLOSE;
+    else
+      return RETUNE;
   }
+  return NORMAL;
 }
