@@ -28,7 +28,7 @@
 #include <vdr/thread.h>
 #include "vnsiserver.h"
 
-static const char *VERSION        = "1.3.1";
+static const char *VERSION        = "1.5.1";
 static const char *DESCRIPTION    = "VDR-Network-Streaming-Interface (VNSI) Server";
 
 extern int PmtTimeout;
@@ -38,18 +38,23 @@ extern int TimeshiftBufferFileSize;
 extern char TimeshiftBufferDir[PATH_MAX];
 extern int PlayRecording;
 extern int AvoidEPGScan;
+extern int DisableScrambleTimeout;
+extern int DisableCamBlacklist;
 
-class cDvbVsniDeviceProbe;
+class cDvbVsniDeviceProbe : public cDvbDeviceProbe
+{
+public:
+  virtual bool Probe(int Adapter, int Frontend);
+};
 
 class cPluginVNSIServer : public cPlugin {
 private:
   cVNSIServer *Server;
   static cPluginVNSIServer *VNSIServer;
-  cDvbVsniDeviceProbe *probe;
+  cDvbVsniDeviceProbe probe;
 
 public:
   cPluginVNSIServer(void);
-  virtual ~cPluginVNSIServer();
   virtual const char *Version(void) { return VERSION; }
   virtual const char *Description(void) { return DESCRIPTION; }
   virtual const char *CommandLineHelp(void);
@@ -72,12 +77,6 @@ public:
   static void StoreSetup(const char *Name, int Value);
 };
 
-class cDvbVsniDeviceProbe : public cDvbDeviceProbe
-{
-public:
-  virtual bool Probe(int Adapter, int Frontend);
-};
-
 class cDvbVnsiDevice : public cDvbDevice
 {
 public:
